import os
import shutil
from pathlib import Path


class Storage:

    root = Path.home()

    def namespace_path(self, namespace):
        return f"{self.root}/.snippets/{namespace}/"

    def commit_path(self, namespace, commit):
        """ Batches are tagged at creation."""
        return f"{self.root}/.snippets/{namespace}/{commit}/"

    def meta_path(self, namespace, commit):
        return f"{self.root}/.snippets/{namespace}/{commit}/meta.json"

    def snippet_path(self, namespace, commit, snippet):
        return f"{self.root}/.snippets/{namespace}/{commit}/{snippet}"

    def write(self, path, data):
        with open(path, "w") as file:
            file.write(data)

    def read(self, path):
        with open(path, "r") as file:
            content = file.read()
        return content

    def delete(self, directory):
        shutil.rmtree(directory)

    def check_exists(self, folder):
        Path(folder).mkdir(parents=True, exist_ok=True)

    def list(self, folder):
        self.check_exists(folder)
        for file in os.listdir(folder):
            if file != "meta.json":
                yield file


_instance = Storage()

namespace_path = _instance.namespace_path
meta_path = _instance.meta_path
read = _instance.read
check_exists = _instance.check_exists
commit_path = _instance.commit_path
snippet_path = _instance.snippet_path
list = _instance.list
delete = _instance.delete
write = _instance.write



import json

from snippet.serializer import DateTimeDecoder
from snippet.serializer import DateTimeEncoder

_GREEN = "\033[92m"
_ENDC = "\033[0m"


class Meta:
    def __init__(self, date, hash, message, count):
        self.date = date
        self.hash = hash
        self.message = message
        self.count = count

    def update(self, message, count):
        self.message = message
        self.count = count

    def __str__(self):
        message = "\n\t".join(self.message.split("\n"))
        return (
            f"{_GREEN}Commit: {self.hash}\n{_ENDC}"
            f"Number of files: {self.count}\n"
            f"Date: {self.date:%a %b %d %H:%M:%S %Y}\n\n"
            f"\t{message}\n"
        )

    def to_json(self):
        return save(
            {
                "date": self.date,
                "message": self.message,
                "count": self.count,
                "hash": self.hash,
            }
        )


def load(content):
    meta = json.loads(content, cls=DateTimeDecoder)
    return Meta(**meta)


def save(dictionary):
    return json.dumps(dictionary, sort_keys=True, cls=DateTimeEncoder)
